import { IsString } from 'class-validator';
import * as jsonwebtoken from 'jsonwebtoken';
import { getConnectionManager } from 'typeorm';
import { IllegalJWTError, JwtNotProvidedError, RefreshTokenCountInvalidError, UserNotFoundError } from '../../errors/AuthError';
import { Logout } from '../entities/Logout';
import { User } from '../entities/User';

export class HandleLogout {
    @IsString()
    token: string;

    public async logout(): Promise<Logout> {
        let logout: Logout = new Logout();
        if (!this.token || this.token === undefined) {
            throw new JwtNotProvidedError()
        }
        let decoded;
        try {
            decoded = jsonwebtoken.verify(this.token, 'securekey')
        } catch (error) {
            throw new IllegalJWTError()
        }
        logout.access_token = this.token
        logout.timestamp = Math.floor(Date.now() / 1000)
        let found_user: User = await getConnectionManager().get().getRepository(User).findOne({ id: decoded["userid"] });
        if (!found_user) {
            throw new UserNotFoundError()
        }
        if (found_user.refreshTokenCount !== decoded["refreshtokencount"]) {
            throw new RefreshTokenCountInvalidError()
        }
        found_user.refreshTokenCount++;
        getConnectionManager().get().getRepository(User).update({ id: found_user.id }, found_user)
        return logout;
    }
}